%global gomodulesmode   GO111MODULE=on

Summary:       Highly-available key value store for configuration and service discovery
Name:          etcd
Version:       3.5.7
Release:       7%{?dist}
License:       Apache-2.0
URL:           https://github.com/etcd-io/etcd
Source0:       %{url}/archive/refs/tags/v%{version}.tar.gz
Source1:       vendor.tar.gz
Source2:       %{name}.conf
Source3:       %{name}.service

#upstream: https://github.com/etcd-io/etcd/pull/15815/files
Patch0001:     fix-CVE-2023-32082.patch
Patch0002:     test-for-CVE-2023-32082.patch
Patch3001:     0001-etcd-3.5.7-add-support-for-loongarch64.patch

BuildRequires: systemd-rpm-macros golang go-rpm-macros
Requires(pre): shadow-utils

%description
etcd is a distributed, consistent key-value store for shared configuration and service discovery,
with a focus on being:

- Simple: well-defined, user-facing API (gRPC)
- Secure: automatic TLS with optional client cert authentication
- Fast: benchmarked 10,000 writes/sec
- Reliable: properly distributed using Raft

%prep
%autosetup -n %{name}-%{version} -a1 -p1
cp -rla vendor/* ./ && rm -r vendor/

%build
mkdir -p ./bin

dir=$(pwd)
for item in server etcdctl etcdutl;do
  cd "$dir/$item"
  %gobuild -o ../bin/"$item"
done

%install
install -D -m 0755 ./bin/server %{buildroot}%{_bindir}/etcd
install -D -m 0755 ./bin/etcdctl %{buildroot}%{_bindir}/etcdctl
install -D -m 0755 ./bin/etcdutl %{buildroot}%{_bindir}/etcdutl

install -D -p -m 0644 %{SOURCE3} %{buildroot}%{_unitdir}/%{name}.service

install -dm 0755 %{buildroot}%{_sysconfdir}/%{name}
install -pm 0644 -t %{buildroot}%{_sysconfdir}/%{name} %{SOURCE2}

install -dm 0755 %{buildroot}%{_sharedstatedir}/%{name}

%pre
getent group %{name} >/dev/null || groupadd -r %{name}
getent passwd %{name} >/dev/null || useradd -r -g %{name} -d %{_sharedstatedir}/%{name} \
    -s /sbin/nologin -c "etcd user" %{name}

%post
%systemd_post %{name}.service

%preun
%systemd_preun %{name}.service

%postun
%systemd_postun %{name}.service

%files
%license LICENSE
%doc CONTRIBUTING.md README.md DCO
%doc etcdctl/README*
%doc etcdutl/README.md
%dir %attr(-,%{name},%{name}) %{_sharedstatedir}/%{name}
%config(noreplace) %{_sysconfdir}/%{name}
%{_unitdir}/%{name}.service
%{_bindir}/etcdctl
%{_bindir}/etcdutl
%{_bindir}/%{name}

%changelog
* Wed Jun 5 2024 Pengda Dou <doupengda@loongson.cn> - 3.5.7-7
- [Type] other
- [DESC] add support for loongarch64

* Wed Apr 24 2024 jackeyji <jackeyji@tencent.com> - 3.5.7-6
- [Type] bugfix
- [DESC] fix start etcd.service failed

* Tue Mar 26 2024 jackeyji <jackeyji@tencent.com> - 3.5.7-5
- fix CVE-2023-32082

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.5.7-4
- Rebuilt for OpenCloudOS Stream 23.09

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.5.7-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.5.7-2
- Rebuilt for OpenCloudOS Stream 23

* Tue Mar 28 2023 Shuo Wang <abushwang@tencent.com> - 3.5.7-1
- initial build

